@@ -733,9 +733,9 @@ def map_data2D_srx(runid, fpath,
     if 'fly' not in plan_n:  # not fly scan
 
         print()
-        print(f"****************************************")
-        print(f"        Loading SRX step scan           ")
-        print(f"****************************************")
+        print("****************************************")
+        print("        Loading SRX step scan           ")
+        print("****************************************")
 
         fly_type = None
 
@@ -829,9 +829,9 @@ def map_data2D_srx(runid, fpath,
     else:
 
         print()
-        print(f"****************************************")
-        print(f"         Loading SRX fly scan           ")
-        print(f"****************************************")
+        print("****************************************")
+        print("         Loading SRX fly scan           ")
+        print("****************************************")
 
         if save_scaler is True:
             scaler_list = ['i0', 'time', 'i0_time', 'time_diff']
@@ -1102,8 +1102,8 @@ def map_data2D_srx(runid, fpath,
 
         print()
         if n_detectors_found == 0:
-            print(f"ERROR: no data from known detectors were found in the database:")
-            print(f"     Check that appropriate fields are included in 'xrf_fly_scan_detector_fields'")
+            print("ERROR: no data from known detectors were found in the database:")
+            print("     Check that appropriate fields are included in 'xrf_fly_scan_detector_fields'")
             print(f"         of configuration file: {config_path}")
         else:
             print(f"Total of {n_detectors_found} detectors were found", end="")
@@ -1207,9 +1207,9 @@ def map_data2D_tes(runid, fpath,
     #        separate file on GPFS, the document contains the path to file)
 
     print()
-    print(f"****************************************")
-    print(f"         Loading TES fly scan           ")
-    print(f"****************************************")
+    print("****************************************")
+    print("         Loading TES fly scan           ")
+    print("****************************************")
 
     xpos_name = 'x_centers'   # For now, we always fly on stage_x (fast axis)
     ypos_name = 'y_centers'
@@ -1324,10 +1324,10 @@ def map_data2D_tes(runid, fpath,
     if output_to_file:
         # output to file
         print(f"Saving data to hdf file #{n_detectors_found}: Detector: {detector_name}.")
-        write_db_to_hdf_base(fpath_out, new_data, metadata=mdata,
-                             fname_add_version=fname_add_version,
-                             file_overwrite_existing=file_overwrite_existing,
-                             create_each_det=create_each_det)
+        fpath_out = write_db_to_hdf_base(fpath_out, new_data, metadata=mdata,
+                                         fname_add_version=fname_add_version,
+                                         file_overwrite_existing=file_overwrite_existing,
+                                         create_each_det=create_each_det)
 
     d_dict = {"dataset": new_data, "file_name": fpath_out,
               "detector_name": detector_name, "metadata": mdata}
